from rapidata.rapidata_client.workflow import Workflow
from rapidata.rapidata_client.order.rapidata_order import RapidataOrder
from rapidata.service import RapidataService


class RapidataOrderBuilder:
    """
    Builder object for creating Rapidata orders.

    Use the fluent interface to set the desired configuration. Add a workflow to the order using `.workflow()` and finally call `.create()` to create the order.

    :param rapidata_service: The RapidataService instance.
    :type rapidata_service: RapidataService
    :param name: The name of the order.
    :type name: str
    """

    def __init__(
        self,
        rapidata_service: RapidataService,
        name: str,
    ):
        self._name = name
        self._rapidata_service = rapidata_service
        self._workflow: Workflow | None = None

    def create(self) -> RapidataOrder:
        """
        Create a RapidataOrder instance based on the configured settings.

        :return: The created RapidataOrder instance.
        :rtype: RapidataOrder
        :raises ValueError: If no workflow is provided.
        """
        if self._workflow is None:
            raise ValueError("You must provide a blueprint to create an order.")

        return RapidataOrder(
            name=self._name, workflow=self._workflow, rapidata_service=self._rapidata_service
        ).create()

    def workflow(self, workflow: Workflow):
        """
        Set the workflow for the order.

        :param workflow: The workflow to be set.
        :type workflow: Workflow
        :return: The updated RapidataOrderBuilder instance.
        :rtype: RapidataOrderBuilder
        """
        self._workflow = workflow
        return self



from typing import Any
from rapidata.rapidata_client.workflow import Workflow


class FreeTextWorkflow(Workflow):
    def __init__(self, question: str):
        super().__init__(type="SimpleWorkflowConfig")
        self._question = question

    def to_dict(self) -> dict[str, Any]:
        return {
            **super().to_dict(),
            "blueprint": {
                "_t": "FreeTextBlueprint",
                "question": self._question,
            },
        }



import os
from rapidata.service import LocalFileService
from rapidata.service import RapidataService


class RapidataDataset:

    def __init__(self, dataset_id: str, rapidata_service: RapidataService):
        self.dataset_id = dataset_id
        self.rapidata_service = rapidata_service
        self.local_file_service = LocalFileService()

    def add_texts(self, texts: list[str]):
        self.rapidata_service.dataset.upload_text_sources(self.dataset_id, texts)

    def add_images_from_paths(self, image_paths: list[str]):
        image_names = [os.path.basename(image_path) for image_path in image_paths]
        images = self.local_file_service.load_images(image_paths)

        self.rapidata_service.dataset.upload_images(self.dataset_id, images, image_names)

    def add_videos_from_paths(self, video_paths: list[str]):
        video_names = [os.path.basename(video_path) for video_path in video_paths]
        videos = self.local_file_service.load_videos(video_paths)

        self.rapidata_service.dataset.upload_videos(self.dataset_id, videos, video_names)


from io import BufferedReader
from PIL import Image
from rapidata.service.rapidata_api_services.base_service import BaseRapidataAPIService
from rapidata.utils.image_utils import ImageUtils


class DatasetService(BaseRapidataAPIService):
    def __init__(self, client_id: str, client_secret: str, endpoint: str):
        super().__init__(
            client_id=client_id, client_secret=client_secret, endpoint=endpoint
        )

    def upload_text_sources(self, dataset_id: str, text_sources: list[str]):
        url = f"{self.endpoint}/Dataset/UploadTextSourcesToDataset"
        payload = {"datasetId": dataset_id, "textSources": text_sources}

        response = self._post(url, json=payload)

        return response

    def upload_images(
        self, dataset_id: str, images: list[Image.Image], image_names: list[str]
    ):
        url = f"{self.endpoint}/Dataset/UploadImagesToDataset"

        params = {"datasetId": dataset_id}

        images_bytes: list[bytes] = [
            ImageUtils.convert_PIL_image_to_bytes(image) for image in images
        ]

        files = [
            ("files", (image_name, image_bytes))
            for image_name, image_bytes in zip(image_names, images_bytes)
        ]

        response = self._post(url, params=params, files=files)

        return response

    def upload_videos(
        self, dataset_id: str, videos: list[BufferedReader], video_names: list[str]
    ):
        url = f"{self.endpoint}/Dataset/UploadImagesToDataset"

        params = {"datasetId": dataset_id}

        files = [
            ("files", (video_name, video_bytes))
            for video_name, video_bytes in zip(video_names, videos)
        ]

        response = self._post(url, params=params, files=files)

        return response

    def upload_images_from_s3(
        self,
        dataset_id: str,
        bucket_name: str,
        region: str,
        source_prefix: str,
        access_key: str,
        secret_key: str,
        clear_dataset: bool = True,
    ):
        url = f"{self.endpoint}/Dataset/UploadFilesFromS3"

        payload = {
            "datasetId": dataset_id,
            "bucketName": bucket_name,
            "region": region,
            "sourcePrefix": source_prefix,
            "accessKey": access_key,
            "secretKey": secret_key,
            "useCustomAwsCredentials": True,
            "clearDataset": clear_dataset,
        }

        response = self._post(url, json=payload)

        return response



class FeatureFlags:
    def __init__(self):
        self._flags: dict[str, str] = {}

    def to_list(self) -> list[dict[str, str]]:
        # transform dict of flags to list of flags
        return [{"key": name, "value": value} for name, value in self._flags.items()]
    
    def alert_on_fast_response(self, value: int):
        self._flags["alertOnFastResponse"] = str(value)
        return self
    
    def disable_translation(self, value: bool):
        self._flags["disableTranslation"] = str(value)
        return self
    
    def free_text_minimum_characters(self, value: int):
        self._flags["freeTextMinimumCharacters"] = str(value)
        return self
    


from .base_referee import Referee as Referee
from .naive_referee import NaiveReferee as NaiveReferee
from .classify_early_stopping_referee import ClassifyEarlyStoppingReferee as ClassifyEarlyStoppingReferee



from typing import Any
from rapidata.rapidata_client.workflow import Workflow


class ClassifyWorkflow(Workflow):
    def __init__(self, question: str, categories: list[str]):
        super().__init__(type="SimpleWorkflowConfig")
        self._question = question
        self._categories = categories

    def to_dict(self) -> dict[str, Any]:
        return {
            **super().to_dict(),
            "blueprint": {
                "_t": "ClassifyBlueprint",
                "title": self._question,
                "possibleCategories": self._categories,
            }
        }



from rapidata.rapidata_client.workflow.referee.base_referee import Referee


class NaiveReferee(Referee):
    """
    The referee defines when a task is considered complete.
    The SimpleReferee is the simplest referee, requiring a fixed number of guesses.
    """

    def __init__(self, required_guesses: int = 10):
        super().__init__()
        self.required_guesses = required_guesses

    def to_dict(self):
        return {
            "_t": "NaiveRefereeConfig",
            "guessesRequired": self.required_guesses,
        }




from typing import Any
from rapidata.service.rapidata_api_services.base_service import BaseRapidataAPIService


class OrderService(BaseRapidataAPIService):
    def __init__(self, client_id: str, client_secret: str, endpoint: str):
        super().__init__(
            client_id=client_id, client_secret=client_secret, endpoint=endpoint
        )

    def create_order(self, name: str, workflow_config: dict[str, Any]) -> tuple[str, str]:
        """
        order_name: name of the order that will be displayed in the Rapidata dashboard.
        question: The question shown to the labeler in the rapid.
        categories: The answer options, between which the labeler can choose.
        target_country_codes: A list of two digit target country codes.
        disable_translation: Per default, the question and categories get translated with DeepL (or Google Translate, if DeepL doesn't support a language). By setting this to `True`, the translation is disabled.
        referee: The referee determines when the task is done. See above for the options.
        """
        url = f"{self.endpoint}/Order/CreateDefaultOrder"

        payload = {
            "orderName": name,
            "datasetName": f"{name} dataset",
            "isPublic": False,
            "workflowConfig": workflow_config,
            "aggregatorType": "Classification",
        }

        response = self._post(url, json=payload)

        return response.json()["orderId"], response.json()["datasetId"]

    def submit(self, order_id: str):
        url = f"{self.endpoint}/Order/Submit"
        params = {"orderId": order_id}

        submit_response = self._post(url, params=params)

        return submit_response
    
    def approve(self, order_id: str):
        url = f"{self.endpoint}/Order/Approve"
        params = {"orderId": order_id}

        approve_response = self._post(url, params=params)

        return approve_response



from rapidata.rapidata_client.order.rapidata_order_builder import RapidataOrderBuilder
from rapidata.service import RapidataService


class RapidataClient:
    """
    A client for interacting with the Rapidata API.
    """

    def __init__(
        self,
        client_id: str,
        client_secret: str,
        endpoint: str = "https://api.rapidata.ai",
    ):
        """
        Initialize the RapidataClient.

        :param client_id: The client ID for authentication.
        :param client_secret: The client secret for authentication.
        :param endpoint: The API endpoint URL. Defaults to "https://api.rapidata.ai".
        """
        self._rapidata_service = RapidataService(
            client_id=client_id, client_secret=client_secret, endpoint=endpoint
        )

    def new_order(self, name: str) -> RapidataOrderBuilder:
        """
        Create a new order using a RapidataOrderBuilder instance.

        :param name: The name of the order.
        :return: A RapidataOrderBuilder instance.
        """
        return RapidataOrderBuilder(rapidata_service=self._rapidata_service, name=name)



from datetime import datetime, timedelta
from typing import Any
import jwt
import requests


class BaseRapidataAPIService:

    def __init__(self, client_id: str, client_secret: str, endpoint: str):
        self.client_id = client_id
        self.client_secret = client_secret
        self.endpoint = endpoint
        self.auth_header = None
        self.token = self._get_auth_token()

    def _check_response(self, response: requests.Response):
        if response.status_code != 200:
            raise Exception(f"Error: {response.status_code} - {response.text}")

    def _get_new_auth_token_if_outdated(self):
        if not self.token or not self._is_token_valid():
            self._get_auth_token()

    def _is_token_valid(self, expiration_threshold: timedelta = timedelta(minutes=5)):
        try:
            payload = jwt.decode(self.token, options={"verify_signature": False})  # type: ignore
            exp_timestamp = payload.get("exp")
            if exp_timestamp:
                expiration_time = datetime.fromtimestamp(exp_timestamp)
                return datetime.now() + expiration_threshold <= expiration_time
        except jwt.DecodeError:
            return False
        return False

    def _get_auth_token(self):
        url = f"{self.endpoint}/Identity/GetClientAuthToken"
        params = {
            "clientId": self.client_id,
        }
        headers = {"Authorization": f"Basic {self.client_secret}"}
        response = requests.post(url, params=params, headers=headers)
        self._check_response(response)
        self.token = response.json().get("authToken")
        if not self.token:
            raise Exception("No token received")
        self.auth_header = {"Authorization": f"Bearer {self.token}"}

    def _post(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        data: dict[str, Any] | None = None,
        json: dict[str, Any] | None = None,
        files: Any | None = None,
    ):
        self._get_new_auth_token_if_outdated()
        response = requests.post(
            url,
            params=params,
            data=data,
            json=json,
            files=files,
            headers=self.auth_header,
        )
        self._check_response(response)
        return response

    def _get(
        self,
        url: str,
        params: dict[str, Any] | None = None,
    ):
        self._get_new_auth_token_if_outdated()
        response = requests.get(url, params=params, headers=self.auth_header)
        self._check_response(response)
        return response



from typing import Any
from rapidata.rapidata_client.workflow import Workflow


class CompareWorkflow(Workflow):
    def __init__(self, criteria: str):
        super().__init__(type="CompareWorkflowConfig")
        self._criteria = criteria
        self._k_factor = 40
        self._match_size = 2
        self._matches_until_completed = 10

    def to_dict(self) -> dict[str, Any]:
        return {
            **super().to_dict(),
            "criteria": self._criteria,
        }
    
    def k_factor(self, k_factor: int):
        self._k_factor = k_factor
        return self
    
    def match_size(self, match_size: int):
        self._match_size = match_size
        return self
    
    def matches_until_completed(self, matches_until_completed: int):
        self._matches_until_completed = matches_until_completed
        return self
    



from rapidata.rapidata_client.order.dataset.rapidata_dataset import RapidataDataset
from rapidata.rapidata_client.workflow import Workflow
from rapidata.service import RapidataService


class RapidataOrder:
    """
    Represents a Rapidata order.

    :param name: The name of the order.
    :type name: str
    :param workflow: The workflow associated with the order.
    :type workflow: Workflow
    :param rapidata_service: The Rapidata service used to create and manage the order.
    :type rapidata_service: RapidataService
    """

    def __init__(
        self, name: str, workflow: Workflow, rapidata_service: RapidataService
    ):
        self.name = name
        self.workflow = workflow
        self.rapidata_service = rapidata_service
        self.order_id = None
        self._dataset = None

    def create(self):
        """
        Creates the order using the provided name and workflow.

        :return: The created RapidataOrder instance.
        :rtype: RapidataOrder
        """
        self.order_id, dataset_id = self.rapidata_service.order.create_order(self.name, self.workflow.to_dict())
        self._dataset = RapidataDataset(dataset_id, self.rapidata_service)
        return self

    def submit(self):
        """
        Submits the order for processing.

        :raises ValueError: If the order has not been created.
        """
        if self.order_id is None:
            raise ValueError("You must create the order before submitting it.")

        self.rapidata_service.order.submit(self.order_id)

    def approve(self):
        """
        Approves the order for execution.

        :raises ValueError: If the order has not been created.
        """
        if self.order_id is None:
            raise ValueError("You must create the order before approving it.")

        self.rapidata_service.order.approve(self.order_id)

    @property
    def dataset(self):
        """
        The dataset associated with the order.

        :raises ValueError: If the order has not been submitted.
        :return: The RapidataDataset instance.
        :rtype: RapidataDataset
        """
        if self._dataset is None:
            raise ValueError("You must submit the order before accessing the dataset.")
        return self._dataset



from rapidata.service.rapidata_api_services.base_service import BaseRapidataAPIService
from rapidata.service.rapidata_api_services.dataset_service import DatasetService
from rapidata.service.rapidata_api_services.order_service import OrderService


class RapidataService(BaseRapidataAPIService):
    def __init__(self, client_id: str, client_secret: str, endpoint: str):
        super().__init__(client_id, client_secret, endpoint)
        self._order_service = OrderService(client_id, client_secret, endpoint)
        self._dataset_service = DatasetService(client_id, client_secret, endpoint)

    @property
    def order(self):
        return self._order_service

    @property
    def dataset(self):
        return self._dataset_service
